import { useState, useEffect } from 'react'
import arrayMove from 'array-move'
import BlockInputs from './BlockInputs'
import { SortableList } from '../../sortable'
import { SubBlockFragment } from '../../gql'
import { TextInput } from '../../form'

const BlockInstanceInputs = ({
  value = [],
  name,
  onChange
}: {
  value?: Partial<SubBlockFragment>[]
  name: string
  onChange: GenericEventHandler
}) => {
  const [state, setState] = useState(value.map(item => item.id))

  function onSortEnd({
    oldIndex,
    newIndex
  }: {
    oldIndex: number
    newIndex: number
  }) {
    const newOrder = arrayMove(state, oldIndex, newIndex)
    setState(newOrder)
    const updatedValues = value.map(item => {
      const order = newOrder.findIndex(orderedId => orderedId === item.id)
      return { ...item, order }
    })
    onChange({
      target: {
        type: 'custom',
        name,
        value: updatedValues
      }
    })
  }

  useEffect(() => {
    const missingIds = value
      .filter(item => !state.includes(item.id))
      .filter(item => item.id && !item.id.startsWith('--'))
      .map(item => item.id)
    const stateWithoutRemovedItems = state.filter(stateId =>
      value.find(item => stateId === item.id)
    )
    setState([...stateWithoutRemovedItems, ...missingIds])
  }, [value])

  const items = state
    .map(stateId => {
      const itemIndex = value.findIndex(item => item.id === stateId)
      if (itemIndex < 0) return null
      const item = value[itemIndex]
      return (
        <div key={item.id}>
          <p>
            {item.order} {item.id}
          </p>
          <TextInput
            name={`${name}.${itemIndex}.rounds`}
            label='Rounds'
            value={item.rounds}
            type='number'
            onChange={data => {
              console.log(data)
              onChange(data)
            }}
          />
          <TextInput
            name={`${name}.${itemIndex}.variation`}
            label='Variation'
            value={item.variation}
            onChange={onChange}
          />
          {item.block && (
            <BlockInputs
              name={`${name}.${itemIndex}.block`}
              value={item.block}
              onChange={onChange}
            />
          )}
          <button
            type='button'
            onClick={ev => {
              const updatedValues: CustomChangeEvent = {
                target: { type: 'custom', name, value }
              }
              if (item.id?.startsWith('++')) {
                updatedValues.target.value = [
                  ...value.slice(0, itemIndex),
                  ...value.slice(itemIndex + 1)
                ]
              } else {
                updatedValues.target.value = [
                  ...value.slice(0, itemIndex),
                  {
                    ...item,
                    id: `--${item.id}`
                  },
                  ...value.slice(itemIndex + 1)
                ]
              }
              console.log(updatedValues)
              onChange(updatedValues)
            }}
          >
            Delete block
          </button>
        </div>
      )
    })
    .filter(block => block !== null)

  return (
    <SortableList
      items={items}
      onSortEnd={onSortEnd}
      useDragHandle
      lockAxis={'y'}
    />
  )
}

export default BlockInstanceInputs
